/*
 *  guasi by Davide Libenzi (generic userspace async syscall implementation)
 *  Copyright (C) 2003  Davide Libenzi
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <poll.h>
#include "guasi.h"
#include "guasi_syscalls.h"




static guasi_param_t guasi_wrap__read(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) read((int) params[0], (void *) params[1],
				    (size_t) params[2]);
}

guasi_req_t guasi__read(guasi_t hctx, void *priv, void *asid, int prio,
			int fd, void *buf, size_t size) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__read, 3,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size);
}

static guasi_param_t guasi_wrap__write(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) write((int) params[0], (void const *) params[1],
				     (size_t) params[2]);
}

guasi_req_t guasi__write(guasi_t hctx, void *priv, void *asid, int prio,
			 int fd, void const *buf, size_t size) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__write, 3,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size);
}

static guasi_param_t guasi_wrap__pread(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) pread((int) params[0], (void *) params[1],
				     (size_t) params[2], (off_t) params[3]);
}

guasi_req_t guasi__pread(guasi_t hctx, void *priv, void *asid, int prio,
			 int fd, void *buf, size_t size, off_t off) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__pread, 4,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) off);
}

static guasi_param_t guasi_wrap__pwrite(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) pwrite((int) params[0], (void const *) params[1],
				      (size_t) params[2], (off_t) params[3]);
}

guasi_req_t guasi__pwrite(guasi_t hctx, void *priv, void *asid, int prio,
			  int fd, void const *buf, size_t size, off_t off) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__pwrite, 4,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) off);
}

static guasi_param_t guasi_wrap__sendfile(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) sendfile((int) params[0], (int) params[1],
					(off_t *) params[2], (size_t) params[3]);
}

guasi_req_t guasi__sendfile(guasi_t hctx, void *priv, void *asid, int prio,
			    int ofd, int ifd, off_t *off, size_t cnt) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__sendfile, 4,
			    (guasi_param_t) ofd, (guasi_param_t) ifd, (guasi_param_t) off,
			    (guasi_param_t) cnt);
}

static guasi_param_t guasi_wrap__recv(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	pfd.fd = (int) params[0];
	pfd.events = POLLIN;
	if (poll(&pfd, 1, params[4]) <= 0)
		return -1;

	return (guasi_param_t) recv((int) params[0], (void *) params[1],
				    (size_t) params[2], (int) params[3]);
}

guasi_req_t guasi__recv(guasi_t hctx, void *priv, void *asid, int prio,
			int fd, void *buf, size_t size, int flags, long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__recv, 5,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) flags, (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__recvfrom(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	pfd.fd = (int) params[0];
	pfd.events = POLLIN;
	if (poll(&pfd, 1, params[6]) <= 0)
		return -1;

	return (guasi_param_t) recvfrom((int) params[0], (void *) params[1],
					(size_t) params[2], (int) params[3],
					(struct sockaddr *) params[4], (socklen_t *) params[5]);
}

guasi_req_t guasi__recvfrom(guasi_t hctx, void *priv, void *asid, int prio,
			    int fd, void *buf, size_t size, int flags,
			    struct sockaddr *from, socklen_t *fromlen, long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__recvfrom, 7,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) flags, (guasi_param_t) from, (guasi_param_t) fromlen,
			    (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__send(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	pfd.fd = (int) params[0];
	pfd.events = POLLOUT;
	if (poll(&pfd, 1, params[4]) <= 0)
		return -1;

	return (guasi_param_t) send((int) params[0], (void const *) params[1],
				    (size_t) params[2], (int) params[3]);
}

guasi_req_t guasi__send(guasi_t hctx, void *priv, void *asid, int prio,
			int fd, void const *buf, size_t size, int flags, long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__send, 5,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) flags, (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__sendto(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	pfd.fd = (int) params[0];
	pfd.events = POLLOUT;
	if (poll(&pfd, 1, params[6]) <= 0)
		return -1;

	return (guasi_param_t) sendto((int) params[0], (void *) params[1],
				      (size_t) params[2], (int) params[3],
				      (struct sockaddr const *) params[4], (socklen_t) params[5]);
}

guasi_req_t guasi__sendto(guasi_t hctx, void *priv, void *asid, int prio,
			  int fd, void const *buf, size_t size, int flags,
			  struct sockaddr const *to, socklen_t tolen, long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__sendto, 7,
			    (guasi_param_t) fd, (guasi_param_t) buf, (guasi_param_t) size,
			    (guasi_param_t) flags, (guasi_param_t) to, (guasi_param_t) tolen,
			    (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__accept(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	pfd.fd = (int) params[0];
	pfd.events = POLLIN;
	if (poll(&pfd, 1, params[3]) <= 0)
		return -1;

	return (guasi_param_t) accept((int) params[0], (struct sockaddr *) params[1],
				      (socklen_t *) params[2]);
}

guasi_req_t guasi__accept(guasi_t hctx, void *priv, void *asid, int prio,
			  int fd, struct sockaddr *addr, socklen_t *addrlen, long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__accept, 4,
			    (guasi_param_t) fd, (guasi_param_t) addr, (guasi_param_t) addrlen,
			    (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__connect(void *priv, guasi_param_t const *params) {
	struct pollfd pfd;

	if (connect((int) params[0], (struct sockaddr *) params[1],
		    (socklen_t) params[2]) != 0) {
		if (errno != EINPROGRESS && errno != EWOULDBLOCK)
			return -1;
		pfd.fd = (int) params[0];
		pfd.events = POLLOUT;
		if (poll(&pfd, 1, params[3]) <= 0)
			return -1;
	}

	return 0;
}

guasi_req_t guasi__connect(guasi_t hctx, void *priv, void *asid, int prio,
			   int fd, struct sockaddr const *addr, socklen_t addrlen,
			   long timeo) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__connect, 4,
			    (guasi_param_t) fd, (guasi_param_t) addr, (guasi_param_t) addrlen,
			    (guasi_param_t) timeo);
}

static guasi_param_t guasi_wrap__open(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) open((char *) params[0], params[1], params[2]);
}

guasi_req_t guasi__open(guasi_t hctx, void *priv, void *asid, int prio,
			char const *name, long flags, long mode) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__open, 3,
			    (guasi_param_t) name, (guasi_param_t) flags, (guasi_param_t) mode);
}

static guasi_param_t guasi_wrap__opendir(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) opendir((char *) params[0]);
}

guasi_req_t guasi__opendir(guasi_t hctx, void *priv, void *asid, int prio,
			   char const *name) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__opendir, 1,
			    (guasi_param_t) name);
}

static guasi_param_t guasi_wrap__readdir(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) readdir((DIR *) params[0]);
}

guasi_req_t guasi__readdir(guasi_t hctx, void *priv, void *asid, int prio,
			   DIR *dir) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__readdir, 1,
			    (guasi_param_t) dir);
}

static guasi_param_t guasi_wrap__stat(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) stat((char const *) params[0], (struct stat *) params[1]);
}

guasi_req_t guasi__stat(guasi_t hctx, void *priv, void *asid, int prio,
			char const *name, struct stat *sbuf) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__stat, 2,
			    (guasi_param_t) name, (guasi_param_t) sbuf);
}

static guasi_param_t guasi_wrap__fstat(void *priv, guasi_param_t const *params) {

	return (guasi_param_t) fstat((int) params[0], (struct stat *) params[1]);
}

guasi_req_t guasi__fstat(guasi_t hctx, void *priv, void *asid, int prio,
			 int fd, struct stat *sbuf) {

	return guasi_submit(hctx, priv, asid, prio, guasi_wrap__fstat, 2,
			    (guasi_param_t) fd, (guasi_param_t) sbuf);
}

